/*
* Project name:
    Pano.vtft
* Generated by:
    Visual TFT
* Date of creation
    01/01/2013
* Time of creation
    5PM
* Created By:
    Andrew Hazelden
* Email:
    andrew@andrewhazelden.com
* Blog:
    http://www.andrewhazelden.com
* Test configuration:
    MCU:             P32MX460F512L
    Dev.Board:       MikroMMB_for_PIC32_hw_rev_1.10
                      http://www.mikroe.com/eng/products/view/595/mikrommb-for-pic32-board/
    Oscillator:      80000000 Hz
    SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/eng/products/view/623/mikroc-pro-for-pic32/
                      
* Notes:
    The pano example draws an interactive panorama onscreen using a series of 2x240 pixel image slices.

    When you drag the view the panorama rotates. If you 'flick' the view with your stylus there is a brief
    'inertia' effect that kicks in.

    When creating an a sliced panorama sequence in VisualTFT use the naming convention image###.bmp 

    The images should be loaded in Visual TFT using the resource collection icon in the
    toolbar. Save the images as an external resource file.

    The Pano_frames.h header file creates the pointer array named "PANO_FRAMES"
    that holds the names of each of the slices frames in the panorama. I looked in the file 
    Pano_resources.h to find out the names that VisualTFT / MikroC used for each of the
    BMP frames.
*/

#include "Pano_objects.h"
#include "Pano_resources.h"
#include "Pano_frames.h"

// Set DEBUG_PANO to 1 to see the diagnostic info or set DEBUG_PANO to 0 to hide the Pano HUD
#define DEBUG_PANO 0

// Turn on /off the inertial glide effect
#define ENABLE_GLIDE 0

// Sets the width in pixels of a panorama slice
#define SLICE_WIDTH 2

// Sets how many slices are visible onscreen at once
// Landscape layout = 160 slices (320 px Screen / 2 px wide slices)

#define VISIBLE_SLICES 160

// This is the glide speed for the Panorama inertia effect. This is visible when the screen is flicked with the stylus.
#define GLIDE_SPEED 6


void writeInt(int n, unsigned int x, unsigned int y);   // Write int values to the TFT screen
void writeLabeledInt(char *label, int n, unsigned int x, unsigned int y); // Write int values to the TFT screen
void writeLabeledInt2(char *label1, int n, char *label2, unsigned int x, unsigned int y);  // Write int values to the TFT screen
int getSlice(int n);          // Keeps slices in range of 0 to FRAME_RANGE
int getSlicePos(int x);       // Slice position on x axis
void getPanoTP();             // Get Panorama touch position
void drawPano(int origin);     // Draw the pano from a starting slice
void showTitles(void);        // Show the title screen

long frame_counter = 0;


void main() {
  Start_TP();                 // Init the touchpanel
  showTitles();               // Show the title screen
  while (1) {
    GetPanoTP();              // Read the touch panel and update the pano
    frame_counter++;
  }
}

// Title Screen
void showTitles(void){
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Write_Text("Mikromedia   Panorama   Viewer   V1.0", 48, 10);  // Write the string on the TFT Screen
  TFT_Write_Text("by   Andrew   Hazelden", 90, 30);          // Write the string on the TFT Screen
  TFT_EXT_Image(84, 63, icon_bmp, 1);                        // Draw the panorama icon
  TFT_Write_Text("Tap   to   Continue...", 105, 210);        // Write the string on the TFT Screen

  // Wait for a user tap to continue the program
  while (!TP_TFT_Press_Detect()) {
    Delay_ms(50);
  }

  // Clear the Screen
  TFT_Fill_Screen(CL_BLACK);
}

// Keeps slices in a valid range of 0 to FRAME_RANGE
int getSlice(int n){
  if(n<0){
    //Keep the current_slice value in the range of 0-FRAME_RANGE
    n = n % (FRAME_RANGE);
    // Handle negative slice numbers
    n = (FRAME_RANGE) + n;
  }
  //Keep the current_slice value in the range of 0-FRAME_RANGE
  n = n % (FRAME_RANGE);
  return n;
}


// Slice position on x axis
int getSlicePos(int x){
  return SLICE_WIDTH * x;
}

// Draw the pano from a starting slice
void drawPano(int origin){
  int current_slice = 0;
  int c=0;
  
  for(c=0;c<=VISIBLE_SLICES;c++){
    current_slice = getSlice(c+origin); // Calculate the active pano slices
    TFT_EXT_Image(getSlicePos(c), 0, PANO_FRAMES[current_slice], 1);   //Draw the current slice
    
    // Debug panorama slice value
    //#if DEBUG_PANO
    //writeInt(current_slice, 0, c * 20);               //Write diagnostic info
    //#endif
  }
  
  // Debug panorama slice value
  #if DEBUG_PANO
  writeLabeledInt("Start Slice:", getSlice(0+origin), 220, 100);               //Write diagnostic info
  writeLabeledInt("End Slice:", getSlice(VISIBLE_SLICES+origin), 220, 120);    //Write diagnostic info
  #endif
}



// Write int values to the TFT screen
void writeInt(int n, unsigned int x, unsigned int y){
  char txt[7];
  IntToStr(n, txt);           // Convert the int to a text string
  TFT_Write_Text(txt, x, y);  // Write the int on the TFT Screen
}

// Write labeled int values to the TFT screen
void writeLabeledInt(char *label, int n, unsigned int x, unsigned int y){
  char txt[7];
  char output_text[110];
  IntToStr(n, txt);           // Convert the int to a text string
  strncpy(output_text, label, 100);  // Add the label to the output_text string
  strncat(output_text, txt, 7);   // Add the number string to the output_text string
  TFT_Write_Text(output_text, x, y);  // Write the string on the TFT Screen
}

// Write labeled int values to the TFT screen
void writeLabeledInt2(char *label1, int n, char *label2, unsigned int x, unsigned int y){
  char txt[7];
  char output_text[110];
  IntToStr(n, txt);                   // Convert the int to a text string
  strncpy(output_text, label1, 50);   // Add the 1st label to the output_text string
  strncat(output_text, txt, 7);       // Add the number string to the output_text string
  strncat(output_text, label2, 50);   // Add the 2nd label to the output_text string
  TFT_Write_Text(output_text, x, y);  // Write the string on the TFT Screen
}



// Get the panorama touchpanel input
void getPanoTP() {
  static int PenDown = 0;          // Was the TP screen pressed?
  static unsigned int x_coord = 0, y_coord = 0;   // Touched Coordinates
  static unsigned int prev_x_coord = 0, prev_y_coord = 0;   // Previous Touched Coordinates
  static int pressed_column = 0;           // The column number on the screen that was tapped
  static int prev_pressed_column = 0;      // The last time the TP column was pressed down
  float view_degrees = 0;                  // View angle in degrees
  const int DEBUG_X_POS = 50;              // X-axis position of debug text
  static int pano_view = 322;                // Current panorama slice position
  static int prev_pano_view = -1;           // Previous panorama slice position
  static int drag_x_distance = 0;          // The length of the TP x-axis drag.
  static int drag_direction = 0;           // Track the direction of the drag
  static int glide_distance = 0;           // Set the glide distance when the drag is released
  
  
  // Keep the pano view within the image slice "frame range"
  pano_view = pano_view % FRAME_RANGE;

  #if ENABLE_GLIDE
  // Run the glide effect when the screen isn't pressed
  if((!TP_TFT_Press_Detect()) && (PenDown == 0)){  
    // Pan the view while there is still 'glide' energy left
    if(glide_distance>=1){
      // Animate the view
      pano_view -= (GLIDE_SPEED*drag_direction);      
      // Reduce the glide distance   
      glide_distance -= GLIDE_SPEED;
    }
    else{
      // Reset the drag direction to 0 when the glide is complete
      drag_direction = 0;
      glide_distance = 0;
    }
  }
  #endif

  

  
  if (TP_TFT_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to display dimensions space
    if (TP_TFT_Get_Coordinates(&x_coord, &y_coord) == 0) {
      //TP is pressed
      if (PenDown == 0) {
        //TP was pressed down
        PenDown = 1;

        //Save the starting column of the TP press
        prev_pressed_column = pressed_column;

        //Save the starting positions of the TP press
        prev_x_coord = x_coord;
        prev_y_coord = y_coord;

        //Reset the drag distance
        drag_x_distance = 0;
        glide_distance = 0;
      }

      //This code runs while the pen is still down
      pressed_column =  x_coord / (SLICE_WIDTH);  // Calculate the active panorama slice column that is pressed
      drag_x_distance = (x_coord - prev_x_coord); // Calculate how far the TP drag event was

      
      // Check if the pano should be updated
      if(x_coord != prev_x_coord){
        //TFT_Fill_Screen(CL_BLACK);              //Clear the Screen
        drawPano(pano_view - (drag_x_distance/SLICE_WIDTH) );  // Draw the pano from the starting TP point
      }
    }
  }
  else if (PenDown == 1) {
    //TP was released
    PenDown = 0;
    
    // Update the pano view if the TouchPannel was dragged   
    pano_view -= (drag_x_distance/SLICE_WIDTH);

    //Check the driection the pano was dragged
    if(drag_x_distance>0){
      drag_direction = 1;      //If the drag distance is positive set the gide direction to 1
    }
    else{
      drag_direction = -1;     //If the drag distance is negative set the gide direction to 0
    }

    #if ENABLE_GLIDE
    // Set the glide distance to part of the distance from the last drag
    //glide_distance = abs(drag_x_distance)/4;
    glide_distance = abs(drag_x_distance)/6;
    #endif
    
    // Reset the drag distance value
    drag_x_distance = 0;
    
    // Reset the TP previous values
    x_coord = 0;
    y_coord = 0;
    prev_y_coord = 0;
    prev_x_coord = 0;
    
    pressed_column = 0;
    prev_pressed_column = 0;
  }
  
  
  // Calculate the current view angle in degrees
  view_degrees = 360 - ( ((float) pano_view / (float) FRAME_RANGE) * 360);
  if(view_degrees<0){
    view_degrees += 360;
  }

  //Update the panorama view if it panned moved more than a slice
  if( prev_pano_view != pano_view ){ 
    
    
    if(pendown){
      drawPano(pano_view - (drag_x_distance/SLICE_WIDTH) );
    }
    else{
      drawPano(pano_view);           // Draw the pano from the starting TP point
    }


    // Debug print the panorama variables
    #if  DEBUG_PANO
    // Screen blanking for debug text
    //TFT_Fill_Screen(CL_BLACK);              //Clear the Screen

    TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
    //writeLabeledInt2( "View:  " , (int)view_degrees % 360, "   degrees" , DEBUG_X_POS, 30);
    writeLabeledInt2( "" , (int)view_degrees % 360, "   deg" , 250, 10);


    TFT_Set_Font(TFT_defaultFont, CL_RED, FO_HORIZONTAL);
    writeLabeledInt( "View  Slice  Number:  " , pano_view, DEBUG_X_POS, 10);

    if (PenDown == 1){
      TFT_Write_Text(  "(TP  Pressed)", DEBUG_X_POS, 50);
    }
    else{
      TFT_Write_Text(  "(TP  Released)", DEBUG_X_POS, 50);
    }
    
    writeLabeledInt( "Pressed   X:  " , x_coord, DEBUG_X_POS, 70);
    //writeLabeledInt( "Prev  X:  " , prev_x_coord, DEBUG_X_POS, 90);
    writeLabeledInt( "Drag  Distance  X:  " , drag_x_distance, DEBUG_X_POS, 110);
    //writeLabeledInt( "Drag  Direction:  " , drag_direction, DEBUG_X_POS, 130);
    //writeLabeledInt( "Glide Distance:  " , glide_distance, DEBUG_X_POS, 150);
    //writeLabeledInt( "Pressed Col:  " , pressed_column, DEBUG_X_POS, 170);
    //writeLabeledInt( "Prev Pressed Col:  " , prev_pressed_column, DEBUG_X_POS, 190);
    #endif
  }
  
  prev_pano_view = pano_view;           // Save the last pano view position
  prev_pressed_column = pressed_column; // Save the last pano position
}